/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <MenuBar.h>
#include <MenuItem.h>
#include <Alert.h>
#include <Font.h>

//#include <iostream>

#include "EBookApp.h"
#include "EBookWindow.h"
#include "EBookView.h"
#include "EBookEngine.h"
#include "SettingsFile.h"
#include "SelectCdRomDlg.h"
#include "PrivateMsg.h"

const char *app_signature = "application/x-vnd.YTWORLD-EBookViewer";

EBBookApp::EBBookApp()
	: BApplication(app_signature)
{
	__settings = NULL;
	__engine   = NULL;
}

EBBookApp::~EBBookApp()
{
	if (__settings) {
		__settings->Save();
		delete __settings;
	}
	if (__engine) {
		delete __engine;
	}
}

void EBBookApp::ReadyToRun()
{
	__engine   = new EBookEngine;
	__settings = new SettingsFile("EBook_Viewer_Settings", "YTWORLD");
	__settings->Load();
	EBookWindow *window = new EBookWindow;
	BMenuBar *menubar = SetupMenuBar();
	window->AddChild(menubar);
	BRect rect(window->Bounds());
	rect.top += menubar->Frame().Height() + 1;
	EBookView *view = new EBookView(__engine, __settings, rect, "");
	__engine->Lock();
	__engine->addObserver(view);
	__engine->Unlock();
	window->AddChild(view);
	window->Show();
}

void EBBookApp::AboutRequested()
{
	BAlert *alert = new BAlert("", "EBook Viewer for BeOS Version 0.0.2\nCopyright © 2000 Y.Takagi All rights reserved", "OK"); 
	alert->SetShortcut(0, B_ESCAPE); 
	alert->Go();
}

void EBBookApp::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
	case M_SELECT_CDROM:
		{
			BMessage m;
			__engine->Lock();
			m.AddString(DICTIONARY_PATH, __engine->getDictionaryPath().c_str());
			__engine->Unlock();
			SelectCdRomDlg *dlg = new SelectCdRomDlg(&m);
			dlg->Show();
		}
		break;

	case M_REPLY_SELECT_CDROM:
		{
			const char *path;
			if (B_OK == msg->FindString(DICTIONARY_PATH, &path)) {
				cout << "path: " << path << endl;
				__engine->Lock();
				__engine->bind(path);
				__engine->notifyObservers();
				__engine->Unlock();
			}
		}
		break;

	case M_WORD_SEARCH_METHOD:
		__engine->Lock();
		__engine->setSearchMethod(EBookEngine::word);
		__engine->Unlock();
		break;

	case M_END_WORD_SEARCH_METHOD:
		__engine->Lock();
		__engine->setSearchMethod(EBookEngine::endword);
		__engine->Unlock();
		break;

	case M_EXACT_WORD_SEARCH_METHOD:
		__engine->Lock();
		__engine->setSearchMethod(EBookEngine::exactword);
		__engine->Unlock();
		break;

	case M_DICTIONARY_SELECTED_ONLY:
		__engine->Lock();
		__engine->setTargetMode(false);
		__engine->Unlock();
		break;

	case M_DICTIONARY_ALL:
		__engine->Lock();
		__engine->setTargetMode(true);
		__engine->Unlock();
		break;

	default:
		BApplication::MessageReceived(msg);
		break;
	}
}

class EBookMenuBar : public BMenuBar {
private:
	EBookEngine *__engine;
public:
	EBookMenuBar(const char *name, EBookEngine *ebe) : BMenuBar(BRect(0, 0, 0, 0), name), __engine(ebe) {}
	virtual void MouseDown(BPoint point)
	{
		BMenuBar::MouseDown(point);
		BMenu *search_menu = SubmenuAt(1);
		int32 index;
		if (search_menu) {
			BMenu *method_menu = search_menu->SubmenuAt(0);
			if (method_menu) {
				__engine->Lock();
				EBookEngine::search_method sm = __engine->getSearchMethod();
				__engine->Unlock();
				switch (sm) {
				case EBookEngine::word:
					index = 0;
					break;
				case EBookEngine::endword:
					index = 1;
					break;
				case EBookEngine::exactword:
					index = 2;
					break;
				}
				BMenuItem *item = method_menu->ItemAt(index);
				if (item) {
					item->SetMarked(true);
				}
			}
			BMenu *dictionary_menu = search_menu->SubmenuAt(1);
			if (dictionary_menu) {
				__engine->Lock();
				bool st = __engine->getTargetMode();
				__engine->Unlock();
				if (st) {
					index = 1;
				} else {
					index = 0;
				}
				BMenuItem *item = dictionary_menu->ItemAt(index);
				if (item) {
					item->SetMarked(true);
				}
			}
		}
	}
};

BMenuBar *EBBookApp::SetupMenuBar()
{
	BMenuBar *menubar = new EBookMenuBar("menubar", __engine);
	BMenu *menu;
	BMenuItem *item;

	menu = new BMenu("File");
	item = new BMenuItem("Select a CD-ROM...", new BMessage(M_SELECT_CDROM));
	menu->AddItem(item);
	item = new BMenuItem("About EBook Viewer...", new BMessage(B_ABOUT_REQUESTED));
	menu->AddItem(item);
	menu->AddSeparatorItem();
	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q');
	menu->AddItem(item);
	menu->SetTargetForItems(this);
	menubar->AddItem(menu);

	BMenu *method_menu = new BMenu("Method");
	item = new BMenuItem("Word", new BMessage(M_WORD_SEARCH_METHOD));
	method_menu->AddItem(item);
	item = new BMenuItem("End-Word", new BMessage(M_END_WORD_SEARCH_METHOD));
	method_menu->AddItem(item);
	item = new BMenuItem("Exact-Word", new BMessage(M_EXACT_WORD_SEARCH_METHOD));
	method_menu->AddItem(item);
	method_menu->SetRadioMode(true);
	method_menu->SetTargetForItems(this);

	BMenu *dictionary_menu = new BMenu("Dictionary");
	item = new BMenuItem("Selected Only", new BMessage(M_DICTIONARY_SELECTED_ONLY));
	dictionary_menu->AddItem(item);
	item = new BMenuItem("All", new BMessage(M_DICTIONARY_ALL));
	dictionary_menu->AddItem(item);
	dictionary_menu->SetRadioMode(true);
	dictionary_menu->SetTargetForItems(this);

	menu = new BMenu("Search");
	menu->AddItem(method_menu);
	menu->AddItem(dictionary_menu);
	menu->SetTargetForItems(this);
	menubar->AddItem(menu);

	return menubar;
}

int main()
{
	EBBookApp app;
	app.Run();
	return 0;
}
