/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <UTF8.h>
#include "EBookEngine.h"

#define MAX_HITS		50
#define MAXLEN_HEADING	127

EBookEngine::EBookEngine()
	: Observable(), __sublistcount(0), __sublist(NULL), __subbook(0),
	__search_method(word), __all_subbooks(false)
{
	::eb_initialize(&__book);
}

EBookEngine::~EBookEngine()
{
	if (__sublist) {
		delete [] __sublist;
	}
	::eb_clear(&__book);
}

bool EBookEngine::bind(const char *path)
{
	bool ret = false;
	Lock();
	::eb_clear(&__book);
	if (::eb_bind(&__book, path) >= 0) {
		int i = getSubBookCount();
		if (i > 0) {
			if (createSubBookList()) {
				if (setSubBook(0)) {
					__path = path;
					setChanged();
					ret = true;
				}
			}
		}
	}
	Unlock();
	return ret;
}

bool EBookEngine::createSubBookList()
{
	bool ret = false;
	Lock();
	if (__sublist) {
		delete [] __sublist;
		__sublist = NULL;
	}
	int count = getSubBookCount();
	if (count > 0) {
		__sublist = new EB_Subbook_Code[count];
		ret = ::eb_subbook_list(&__book, __sublist) >= 0;
	}
	Unlock();
	return ret;
}

int EBookEngine::getSubBookCount()
{
	Lock();
	__sublistcount = ::eb_subbook_count(&__book);
	Unlock();
	return __sublistcount;
}

char *EBookEngine::getSubBookTitle2(int i)
{
	char *buf = NULL;
	Lock();
	if (__sublist && i < __sublistcount) {
		const char *title = ::eb_subbook_title2(&__book, __sublist[i]);
		int32 state     = 0;
		int32 srcLength = strlen(title);
		int32 dstLength = srcLength * 2;
		buf = new char[dstLength + 1];
		::convert_to_utf8(B_EUC_CONVERSION, title, &srcLength, buf, &dstLength, &state);
		buf[dstLength] = '\0';
	}
	Unlock();
	return buf;
}

bool EBookEngine::setSubBook(int i)
{
	bool ret = false;
	Lock();
	if (__sublist && i < __sublistcount) {
		__subbook = i;
		ret = ::eb_set_subbook(&__book, __sublist[i]) >= 0;
	}
	Unlock();
	return ret;
}

int EBookEngine::enumTitle(PFN_add_title pfn_add_title, void *param)
{
	int count = 0;
	EB_TitleInfo ebti;
	Lock();
	count = getSubBookCount();
	for (int i = 0 ; i < count ; i++) {
		ebti.index = i;
		ebti.title = getSubBookTitle2(i);
		(*pfn_add_title)(&ebti, param);
		delete [] ebti.title;
	}
	Unlock();
	return count;
}

void EBookEngine::search(
	const char *word,
	PFN_add_heading pfn_add_heading,
	void *param)
{
	bool ret = false;
	int32 state     = 0;
	int32 srcLength = strlen(word);
	int32 dstLength = srcLength;
	char *buf = new char[dstLength + 1];
	::convert_from_utf8(B_EUC_CONVERSION, word, &srcLength, buf, &dstLength, &state);
	buf[dstLength] = '\0';

	Lock();

	if (__all_subbooks) {
		int count = getSubBookCount();
		int subbook = getSubBook();
		for (int i = 0 ; i < count ; i++) {
			setSubBook(i);
			__search(buf, pfn_add_heading, param);
		}
		setSubBook(subbook);
	} else {
		__search(buf, pfn_add_heading, param);
	}

	Unlock();
	delete [] buf;
}

void EBookEngine::__search(
	const char *word,
	PFN_add_heading pfn_add_heading,
	void *param)
{
	int search_done;
	switch (getSearchMethod()) {
	case endword:	
		search_done = ::eb_search_endword(&__book, word);
		break;
	case exactword:
		search_done = ::eb_search_exactword(&__book, word);
			break;
	default:
		search_done = ::eb_search_word(&__book, word);
		break;
	}

	if (search_done >= 0) {
		EB_Hit hits[MAX_HITS];
		char heading[MAXLEN_HEADING + 1];
		EB_HeadingInfo ebhf;
		ebhf.subbook = getSubBook();

		while (1) {
			int hitcount = ::eb_hit_list(&__book, hits, MAX_HITS);
			if (hitcount <= 0) {
				break;
			}
			for (int i = 0 ; i < hitcount ; i++) {
				if (::eb_seek(&__book, &(hits[i].heading)) >= 0) {
					if (::eb_heading(&__book, NULL, NULL, heading, MAXLEN_HEADING) >= 0) {
						int32 state     = 0;
						int32 srcLength = strlen(heading);
						int32 dstLength = srcLength * 2;
						char *buf = new char[dstLength + 1];
						::convert_to_utf8(B_EUC_CONVERSION, heading, &srcLength, buf, &dstLength, &state);
						buf[dstLength] = '\0';
						ebhf.heading   = buf;
						ebhf.content   = &(hits[i].text);
						(*pfn_add_heading)(&ebhf, param);
						delete [] buf;
					}
				}
			}
		}
	}
}

char *EBookEngine::copyright()
{
	char *buf = NULL;
	EB_Position position;
	char text[EB_SIZE_PAGE + 1];
	Lock();
	if (::eb_copyright(&__book, &position) >= 0) {
		if (::eb_seek(&__book, &position) >= 0) {
			if (::eb_text(&__book, NULL, NULL, text, EB_SIZE_PAGE) >= 0) {
				int32 state     = 0;
				int32 srcLength = strlen(text);
				int32 dstLength = srcLength * 2;
				buf = new char[dstLength + 1];
				::convert_to_utf8(B_EUC_CONVERSION, text, &srcLength, buf, &dstLength, &state);
				buf[dstLength] = '\0';
			}
		}
	}
	Unlock();
	return buf;
}

char *EBookEngine::getContent(const EB_Position *pos, int subbook)
{
	char *buf = NULL;
	char text[EB_SIZE_PAGE + 1];
	Lock();

	int current_subbook = getSubBook();
	if ((subbook >= 0) && (current_subbook != subbook)) {
		setSubBook(subbook);
	}

	if (::eb_seek(&__book, pos) >= 0) {
		if (::eb_text(&__book, NULL, NULL, text, EB_SIZE_PAGE) >= 0) {
			int32 state     = 0;
			int32 srcLength = strlen(text);
			int32 dstLength = srcLength * 2;
			buf = new char[dstLength + 1];
			::convert_to_utf8(B_EUC_CONVERSION, text, &srcLength, buf, &dstLength, &state);
			buf[dstLength] = '\0';
		}
	}

	if ((subbook >= 0) && (current_subbook != subbook)) {
		setSubBook(current_subbook);
	}

	Unlock();
	return buf;
}

string EBookEngine::getDictionaryPath()
{
	string s;
	Lock();
	s = __path;
	Unlock();
	return s;
}

int EBookEngine::getSubBook() const
{
	return __subbook;
}

void EBookEngine::setTargetMode(bool f)
{
	Lock();
	__all_subbooks = f;
	Unlock();
}

bool EBookEngine::getTargetMode() const
{
	return __all_subbooks;
}

void EBookEngine::setSearchMethod(search_method method)
{
	Lock();
	__search_method = method;
	Unlock();
}

EBookEngine::search_method EBookEngine::getSearchMethod() const
{
	return __search_method;
}
