/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <Window.h>
#include <Button.h>
#include <TextControl.h>
#include <ListView.h>
#include <ListItem.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <ScrollView.h>
#include <Dragger.h>
#include <Entry.h>
#include <Path.h>
#include <UTF8.h>

//#include <iostream>
#include <string>

#include "EBookApp.h"
#include "EBookView.h"
#include "EBookEngine.h"
#include "SettingsFile.h"

enum MSGS {
	M_SELSUBBOOK = 1,
	M_SEARCH,
	M_SELLISTITEM,
	M_UPDATE
};

const char *DICTIONARY_LIST_TEXT = "Dictionary:";
const char *STRING_TEXT	         = "Word:";
const char *FIND_BUTTON_TEXT     = "Search";
const char *DICTIONARY_PATH      = "dictionary_path";
const char *SUBBOOK_INDEX        = "subbook_index";
const char *SEARCH_METHOD        = "search_method";
const char *SEARCH_TARGET        = "search_target";

class EBookStringItem : public BStringItem {
public:
	int subbook;
	EB_Position position;
	EBookStringItem(const EB_HeadingInfo *ebhf)
		: BStringItem(ebhf->heading), subbook(ebhf->subbook), position(*ebhf->content)
	{
	}
};

class EBookDragger : public BDragger {
public:
	EBookDragger(BRect frame, BView *target, uint32 resizingMode = B_FOLLOW_NONE, uint32 flags = B_WILL_DRAW)
		: BDragger(frame, target, resizingMode, flags)
	{
	}
	virtual void Draw(BRect rect)
	{
		SetHighColor(Target()->ViewColor());
		FillRect(rect, B_SOLID_HIGH);
		BDragger::Draw(rect);
	}
};

EBookView::EBookView(EBookEngine *engine, BMessage *message, BRect frame, const char *name, uint32 resizingMode, uint32 flags) 
	: BView(frame, "", resizingMode, flags), Observer()
{
	replicant    = false;
	__text       = NULL;
	__dictionary = NULL;
	__listview   = NULL;
	__content    = NULL;
	__message    = message;
	__engine     = engine;
}

EBookView::EBookView(BMessage *data)
	: BView(data), Observer()
{
	replicant    = true;
	__text       = NULL;
	__dictionary = NULL;
	__listview   = NULL;
	__content    = NULL;
	__message    = new BMessage(*data);
	__engine     = new EBookEngine;
	__engine->addObserver(this);
}

EBookView::~EBookView()
{
	if (replicant) {
		delete __engine;
		delete __message;
	} else {
		Save(__message);
	}
}

void EBookView::Load(BMessage *msg)
{
	const char *dictionary_path;
	if (B_OK != msg->FindString(DICTIONARY_PATH, 0, &dictionary_path)) {
		dictionary_path = "";
	}

	int32 subbook = 0;
	if (B_OK != msg->FindInt32(SUBBOOK_INDEX, 0, &subbook)) {
		subbook = 0;
	}

	int32 method = 0;
	if (B_OK != msg->FindInt32(SEARCH_METHOD, 0, &method)) {
		method = 0;
	}

	bool target = false;
	if (B_OK != msg->FindBool(SEARCH_TARGET, 0, &target)) {
		target = false;
	}

	__engine->setSearchMethod((EBookEngine::search_method)method);
	__engine->setTargetMode(target);

	if (__engine->bind(dictionary_path)) {
		__engine->setSubBook(subbook);
		InitView();
	}
}

void EBookView::Save(BMessage *msg) const
{
	const char *path = __engine->getDictionaryPath().c_str();
	if (msg->HasString(DICTIONARY_PATH))
		msg->ReplaceString(DICTIONARY_PATH, path);
	else
		msg->AddString(DICTIONARY_PATH, path);

	int32 subbook = __engine->getSubBook();
	if (msg->HasInt32(SUBBOOK_INDEX))
		msg->ReplaceInt32(SUBBOOK_INDEX, subbook);
	else
		msg->AddInt32(SUBBOOK_INDEX, subbook);

	int32 method = (int32)__engine->getSearchMethod();
	if (msg->HasInt32(SEARCH_METHOD))
		msg->ReplaceInt32(SEARCH_METHOD, method);
	else
		msg->AddInt32(SEARCH_METHOD, subbook);

	bool target = __engine->getTargetMode();
	if (msg->HasBool(SEARCH_TARGET))
		msg->ReplaceBool(SEARCH_TARGET, target);
	else
		msg->AddBool(SEARCH_TARGET, target);
}

BArchivable *EBookView::Instantiate(BMessage *msg)
{
	if (validate_instantiation(msg, "EBookView")) {
		return new EBookView(msg);
	}
	return NULL;
}

status_t EBookView::Archive(BMessage *msg, bool deep) const
{
	BView::Archive(msg, deep);
	msg->AddString("add_on", app_signature);
	msg->AddString("class", "EBookView");
	Save(msg);
	return B_OK;
}

void EBookView::AttachedToWindow()
{
	const BRect DICTIONARY_LIST_RECT ( 10,  10, 200,  30);
	const BRect STRING_RECT          ( 10,  40, 200,  60);
	const BRect FIND_BUTTON_RECT     ( 10,  70, 200,  90);
	const BRect FOUND_LIST_RECT      ( 10, 110, 200 - B_V_SCROLL_BAR_WIDTH,                 Bounds().bottom - 10);
	const BRect CONTENT_RECT         (215,  10, Bounds().right - B_V_SCROLL_BAR_WIDTH - 10, Bounds().bottom - 10);

	__dictionary = new BPopUpMenu("");
	__dictionary->SetRadioMode(true);
	BMenuField *menufield = new BMenuField(DICTIONARY_LIST_RECT, "", DICTIONARY_LIST_TEXT, __dictionary);
	AddChild(menufield);
	float width = StringWidth(DICTIONARY_LIST_TEXT) + 10;
	menufield->SetDivider(width);

	__text = new BTextControl(STRING_RECT, "", STRING_TEXT, "", NULL);
	AddChild(__text);
	__text->SetDivider(StringWidth(STRING_TEXT) + 8);

	BButton *button = new BButton(FIND_BUTTON_RECT, "", FIND_BUTTON_TEXT, new BMessage(M_SEARCH));
	AddChild(button);
	button->MakeDefault(true);
	button->SetTarget(this);

	uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM;
	__listview = new BListView(FOUND_LIST_RECT, "", B_SINGLE_SELECTION_LIST, resizingMode);
	AddChild(new BScrollView("", __listview, resizingMode, 0, false, true));
	__listview->SetSelectionMessage(new BMessage(M_SELLISTITEM));
	__listview->SetTarget(this);

	BRect textRect(CONTENT_RECT);
	textRect.OffsetTo(0, 0);
	resizingMode = B_FOLLOW_ALL;
	__content = new BTextView(CONTENT_RECT, "", textRect, resizingMode, B_WILL_DRAW);
	AddChild(new BScrollView("", __content, resizingMode, 0, false, true));
/*
	BRect dragRect(Bounds());
	dragRect.top   = dragRect.bottom - 10;
	dragRect.right = 10;
	BDragger *dragger = new EBookDragger(dragRect, this, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	AddChild(dragger);
*/

/*
	int32 count = CountChildren();
	for (int32 i = 0 ; i < count ; i++) {
		BView *v = ChildAt(0); 
		if (v) {
			cout << "ggg: " << i << endl;
			BFont font;
			v->GetFont(&font);
			font.SetSize(20.0);
			v->SetFont(&font);
		}
	}
*/

	Load(__message);
}

static void add_title(const EB_TitleInfo *ebti, void *param)
{
	BPopUpMenu *menu = (BPopUpMenu *)param;
	menu->AddItem(new BMenuItem(ebti->title, new BMessage(M_SELSUBBOOK)));
}

void EBookView::EmptyListView()
{
	int32 count = __dictionary->CountItems();
	if  (count > 0) {
		BMenuItem *item;
		while (count--) {
			item = __dictionary->RemoveItem(count);
			if (item) {
				delete item;
			}
		}
	}
}

void EBookView::InitView()
{
	int32 count;
	BMenuItem *item;

	EmptyListView();

	count = __engine->enumTitle(add_title, __dictionary);
	__dictionary->SetTargetForItems(this);
	int subbook = __engine->getSubBook();
	if (subbook != -1) {
		item = __dictionary->ItemAt(subbook);
		if (item) {
			item->SetMarked(true);
		}
		__text->SetText("");
		__listview->MakeEmpty();
		char *text = __engine->copyright();
		if (text) {
			__content->SetText(text);
			delete [] text;
		}
	}
}

static void add_heading(const EB_HeadingInfo *ebif, void *param)
{
	BListView *listview = (BListView *)param;
	listview->AddItem(new EBookStringItem(ebif));
}

void EBookView::MessageReceived(BMessage *msg)
{
/*
	if (msg->WasDropped()) {
		entry_ref ref;
		msg->FindRef("refs", 0, &ref);
		BPath path(&ref);
		if (__engine->bind(path.Path())) {
			__engine->setSubBook(0);
			InitView();
		}
		return;
	}
*/
	switch (msg->what) {
	case M_SELSUBBOOK:
	case M_SEARCH:
		{
			__listview->MakeEmpty();
			BMenuItem *menuItem = __dictionary->FindMarked();
			if (menuItem) {
				int subindex = __dictionary->IndexOf(menuItem);
				if (__engine->setSubBook(subindex)) {
					if (__text->Text() && *__text->Text() != '\0') {
						__content->SetText("");
						__engine->search(__text->Text(), add_heading, __listview);
						if (__listview->CountItems() > 0) {
							__listview->Select(0);
						}
					} else {
						char *text = __engine->copyright();
						__content->SetText(text);
						delete [] text;
					}
				}
			}
		}
		break;

	case M_SELLISTITEM:
		{
			int32 selected = __listview->CurrentSelection(0);
			if (selected >= 0) {
				EBookStringItem *item = (EBookStringItem *)__listview->ItemAt(selected);
				if (item) {
					char *text = __engine->getContent(&item->position, item->subbook);
					__content->SetText(text);
					delete [] text;
				}
			}
		}
		break;

	case M_UPDATE:
		InitView();
		break;

	default:
		BView::MessageReceived(msg);
		break;
	}
}

void EBookView::update(Observable *, void *)
{
	Window()->PostMessage(new BMessage(M_UPDATE), this);
}
